
/*
 * $Id: NotifMsg.cc,v 1.1.2.6 1999/06/14 20:59:23 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "NetAddr.h"
#include "NotifMsg.h"
#include "gadgets.h"

/* network <-> host byte order conversions */

inline
void hton(Time &t) {
	t.tv_sec = htonl(t.tv_sec);
	t.tv_usec = htonl(t.tv_usec);
}

inline
void ntoh(Time &t) {
	t.tv_sec = ntohl(t.tv_sec);
	t.tv_usec = ntohl(t.tv_usec);
}

inline
void hton(NetDouble &d) {
	d.mnt = (int) htonl(d.mnt);
	d.exp = (int) htonl(d.exp);
}

inline
void ntoh(NetDouble &d) {
	d.mnt = (int) ntohl(d.mnt);
	d.exp = (int) ntohl(d.exp);
}

/* NotifMsg */

void NotifMsg::hton() {
	theId = (int) htonl(theId);
	theSize = (int) htonl(theSize);
}

void NotifMsg::ntoh() {
	theId = (int) ntohl(theId);
	theSize = (int) ntohl(theSize);
}


/* StatusNotifMsg */

StatusNotifMsg::StatusNotifMsg(const String &aLabel): NotifMsg(mtStatus) {
	theSize = sizeof(*this);

	memset(theLabel, 0, sizeof(theLabel));
	if (aLabel)
		memcpy(theLabel, aLabel, Min(aLabel.len(), (int)sizeof(theLabel)-1));
	
	theXactCnt = theSockCnt = -1;
}

void StatusNotifMsg::hton() {
	NotifMsg::hton();

	::hton(theStartTime);
	::hton(theSndTime);
	::hton(theRespTime);

	::hton(theTput);
	::hton(theBwidth);
	::hton(theDHR);

	theXactCnt = (int) htonl(theXactCnt);
	theSockCnt = (int) htonl(theSockCnt);
}

void StatusNotifMsg::ntoh() {
	NotifMsg::ntoh();

	::ntoh(theStartTime);
	::ntoh(theSndTime);
	::ntoh(theRespTime);

	::ntoh(theTput);
	::ntoh(theBwidth);
	::ntoh(theDHR);

	theXactCnt = (int) ntohl(theXactCnt);
	theSockCnt = (int) ntohl(theSockCnt);
}


/* StatusFwdMsg */

StatusFwdMsg::StatusFwdMsg(): theCopyCnt(-1) {
	theSndAddr.addr.s_addr = INADDR_NONE;
	theSndAddr.port = -1;
}

StatusFwdMsg::StatusFwdMsg(const StatusNotifMsg &m, Time aRcvTime, const NetAddr &aSndAddr):
	StatusNotifMsg(m), theRcvTime(aRcvTime), theCopyCnt(-1) {

	theSndAddr.addr = aSndAddr.addrN();
	theSndAddr.port = aSndAddr.port();
}

void StatusFwdMsg::hton() {
	StatusNotifMsg::hton();

	::hton(theRcvTime);

	// addr_in is always in network byte order?
	theSndAddr.port = (int) htonl(theSndAddr.port);

	theCopyCnt = (int) htonl(theCopyCnt);
}

void StatusFwdMsg::ntoh() {
	StatusNotifMsg::ntoh();

	::ntoh(theRcvTime);

	// addr_in is always in network byte order?
	theSndAddr.port = (int) ntohl(theSndAddr.port);

	theCopyCnt = (int) ntohl(theCopyCnt);
}
		
