
/*
 * $Id: RndPermut.cc,v 1.1 1999/05/26 19:03:19 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <limits.h>

#include "Rnd.h"
#include "RndPermut.h"

static int *ThePermTable = 0;
static int ThePermTableCap = 0;

void RndPermutConfig(int setSize, int seed) {
	delete ThePermTable;

	if (setSize > 0) {
		ThePermTable = new int[setSize];
		ThePermTableCap = setSize;
		RndPermutSeed(seed);
	} else {
		ThePermTable = 0;
		ThePermTableCap = 0;
	}
}

// feel set with "good" values
void RndPermutSeed(int seed) {
	RndGen rng(seed);
	for (int i = 0; i < ThePermTableCap; ++i)
		ThePermTable[i] = rng.ltrial();
}

inline
int PermItem(int i) {
	return i >= 0 ?
		ThePermTable[i % ThePermTableCap] :
		ThePermTable[(i + INT_MAX) % ThePermTableCap];
}

int RndPermut(int n, int m) {
	const int a = n / ThePermTableCap;
	const int b = n % ThePermTableCap;
	const int offset = PermItem(a) - m;
	return PermItem(offset + b);
}

/* initialization */

int RndPermutfInit::theUseCount = 0;

void RndPermutfInit::init() {
	Assert(theUseCount == 1);
	RndPermutConfig(1048573); // 82,025-th prime number! (1M=1048576)
}

void RndPermutfInit::clean() {
	Assert(theUseCount == 0);
	RndPermutConfig(0);
}
