
/*
 * $Id: OLog.h,v 1.1.1.5 1999/03/08 07:56:11 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef O_LOG_CLASS_H
#define O_LOG_CLASS_H

#include "Time.h"
#include "Size.h"
#include "NetDouble.h"
#include "String.h"
#include "Array.h"

class ostream;

// manipulators to start and end an entry
class bege {
	public:
		bege(int aTag): theTag(aTag) {}
		int tag() const { return theTag; }

	protected:
		int theTag;
};

// binary two-stage (memory -> file) output log
class OLog {
	protected:
		typedef void (Manip)(class OLog &);

	public:
		OLog();
		virtual ~OLog();

		Size capacity() const { return theCapacity; }
		Size size() const { return theSize; }

		void stream(const String &aFileName, ostream *aStream);
		void resize(Size aCap);

		OLog &operator <<(int x) { puti(x); return *this; }
		OLog &operator <<(double x) { NetDouble nd(x); puti(nd.mnt); puti(nd.exp); return *this; }
		OLog &operator <<(char c) { putS(&c, 1); return *this; }
		OLog &operator <<(const char *s) { putS(s, strlen(s)+1); return *this; }
		OLog &operator <<(const String &s) { putS(s.str(), s.len()+1); return *this; }
		OLog &operator <<(const Time &t) { puti(t.tv_sec); puti(t.tv_usec); return *this; }

		OLog &operator <<(const bege &b) { begEntry(b.tag()); return *this; }
		OLog &operator <<(Manip m) { m(*this); return *this; }

		void puti(int x) { x = htonl(x); put(&x, sizeof(x)); }
		void puti(const int *x, int count);
		void putS(const char *s, int size) { puti(size); put(s, size); }

		virtual void begEntry(int tag);
		void endEntry();

		void close(); // flushes if needed

	protected:
		void putHeader();
		inline void put(const void *buf, int size);
		void flush();

	protected:
		ostream *theStream;
		String theFileName;

		char *theBuf;
		Size theCapacity; // allocated
		Size theSize;     // used

		char *theEntry;	  // start of a current entry
		Time theModTime;  // time when last entry was recorded

		bool isFull;      // true if an overflow condition was detected
};

inline void ende(OLog &log) { log.endEntry(); }

inline
void OLog::put(const void *buf, int size) {
	if (size > 0) {
		Assert(buf);
		// ignore if already full or if new entry does not fit
		if (!isFull && theSize + size <= theCapacity) {
			memcpy(theBuf + theSize, buf, size);
			theSize += size;
		} else {
			isFull = true;
		}
	}
}

template <class Item>
inline
OLog &operator <<(OLog &log, const Array<Item> &a) {
	log << a.count();
	for (int i = 0; i < a.count(); ++i)
		log << a[i];
	return log;
}

#endif
