
/*
 * $Id: StatPing.h,v 1.3 1999/05/24 17:41:42 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef STAT_PING_H
#define STAT_PING_H

#include "BigSize.h"
#include "LevelStat.h"
#include "StatInt.h"


// handy for keeping track of hits and misses
class HRStat {
	public:
		HRStat();

		void reset();
		OLog &store(OLog &log) const;
		ILog &load(ILog &);

		void hit(Size sz) { ++theHitCnt; if (sz > 0) theHitSz += sz; }
		void miss(Size sz) { ++theMissCnt; if (sz > 0) theMissSz += sz; }

		int hits() const { return theHitCnt; }
		int misses() const { return theMissCnt; }
		int xacts() const { return theHitCnt + theMissCnt; }
		double dhr() const;
		double bhr() const;
		double dhp() const;
		double bhp() const;

		HRStat &operator +=(const HRStat &s);

	protected:
		int theHitCnt;
		int theMissCnt;
		BigSize theHitSz;
		BigSize theMissSz;
};

inline OLog &operator <<(OLog &log, const HRStat &s) { return s.store(log); }
inline ILog &operator >>(ILog &log, HRStat &s) { return s.load(log); }


// Accumulates "cheap" statistics (in StatPingRec) and dumps it every period
class StatPingRec {
	public:
		StatPingRec();

		void restart();
		OLog &store(OLog &log) const;
		ILog &load(ILog &);

		void xactBeg() { ++theXactLvl; }
		void xactEnd() { --theXactLvl; }
		void xactErr() { theXactLvl.failure(); }

		void realHit(Size sz) { theRealHR.hit(sz); }
		void realMiss(Size sz) { theRealHR.miss(sz); }

		void idealHit(Size sz) { theIdealHR.hit(sz); }
		void idealMiss(Size sz) { theIdealHR.miss(sz); }

		void chbObj(Size sz, bool cachable) { cachable ? theChbR.hit(sz) : theChbR.miss(sz); }

		void connOpen() { ++theOpenLvl; }
		void connEstb() { ++theEstbLvl; }
		void connClose() { --theEstbLvl; --theOpenLvl; }

		void selected() { ++theSelCnt; }
		void read(Size sz) { ++theRdCnt; if (sz > 0) theRdSz += sz; }
		void wrote(Size sz) { ++theWrCnt; if (sz > 0) theWrSz += sz; }

		void err(int /*no*/) { ++theErrCnt; }

		double reqRate() const; // req/sec
		double repRate() const; // rep/sec

		StatPingRec &operator +=(const StatPingRec &s);

	public: /* read only */
		LevelStat theXactLvl; // active xactions
		LevelStat theOpenLvl; // open connections
		LevelStat theEstbLvl; // estanlished connections

		int theSelCnt;  // io
		int theRdCnt;
		int theWrCnt;
		Size theRdSz;
		Size theWrSz;

		HRStat theRealHR;   // measured HR
		HRStat theIdealHR;  // offered HR
		HRStat theChbR;     // cachablity ratio

		int theErrCnt;    // other

		Time theDuration; // actual int duration, computed in flush()

};

class StatPing: public StatClockInt, public StatPingRec {
	public:
		StatPing(Time aPeriod);

		virtual void flush();

	protected:
		virtual void restart();
		void report() const;
};

// XXX: should this be a template?
inline OLog &operator <<(OLog &log, const StatPingRec &s) { return s.store(log); }
inline ILog &operator >>(ILog &log, StatPingRec &s) { return s.load(log); }

#endif
