
/*
 * $Id: StatSoft.h,v 1.2 1999/06/02 06:26:38 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef STAT_SOFT_H
#define STAT_SOFT_H

#include "StatInt.h"
#include "histograms.h"

// Accumulates "expensive" stats (in StatSoftRec) and dumps them every So OFTen

class StatSoftRec {
	public:
		void restart();
		void store(OLog &log) const;
		void load(ILog &);

		inline void xactLifeTime(const Time &tm, int xc);
		inline void connLifeTime(const Time &tm, int useCnt);

		StatSoftRec &operator +=(const StatSoftRec &s);

		const TimeHist &xactLifeTime() const { return theXactLifeTime; }
		const TimeHist &connLifeTime() const { return theConnLifeTime; }
		const Log2Hist &connUse() const { return theConnUse; }
		const TimeVsClassHist &xactLifeVsClass() const { return theXactLifeVsClass; }

		// XXX: msec
		double xactLifeTimeMean() const { return xactLifeTime().stats().mean(); }

	protected:
		TimeHist theXactLifeTime;
		TimeHist theConnLifeTime;
		Time theDuration;    // actual duration computed in flush()

		Log2Hist theConnUse;
		TimeVsClassHist theXactLifeVsClass;
};

class StatSoft: public StatClockInt, public StatSoftRec {
	public:
		StatSoft(Time aPeriod);

		virtual void flush();

	protected:
		virtual void restart();
};


/* inlined methods */

inline
void StatSoftRec::xactLifeTime(const Time &tm, int xc) {
	theXactLifeTime.record(tm);
	theXactLifeVsClass.record(tm, xc);
}

inline
void StatSoftRec::connLifeTime(const Time &tm, int useCnt) {
	theConnLifeTime.record(tm);
	theConnUse.record(useCnt);
}


#endif
